/**
* Maintains the heap structure of a given array.
<pre>
An array A[lo:hi] is said to satisfy the "heap property" if
for i = 0,.., hi - lo:
A[lo + i] <= A[lo + 2 * i + 1]  and
A[lo + i] < = A[lo + 2 * i + 2],
whenever these indices are in the range [lo..hi].
</pre>
* <p> Copyright 2000 - Dung X. Nguyen - All rights reserved. </p>
*/
public class Heapifier {
    public static final Heapifier Singleton = new Heapifier ();
    private Heapifier() {
    }

    /**
    * "Sifts" A[cur] up the array A to maintain the heap property.
    * @param A A[lo:cur-1] is a heap.
    * @param lo the low index of A.
    * @param cur lo <= cur <= the high index of A.
    */
    public void siftUp(IOrdered[] A, int lo, int cur) {
        IOrdered dat = A[cur];
        int parent = (cur - lo - 1) / 2 + lo;  // index of parent.
        while (0 < (cur - lo) && (dat.compare(A[parent])==IOrdered.GREATER)) {
            A[cur] = A[parent];
            cur = parent;
            parent = (cur - lo - 1) / 2 + lo;
        }
        A[cur] = dat;
    }


    /**
    * "Sifts" the value at A[cur] down the array A to maintain the heap property.
    * @param A  A[cur + 1:hi] satisfies the heap property.
    * @param lo the low index of A.
    * @param cur lo <= cur <= hi.
    * @param hi  the high index of a.
    */
    public void siftDown(IOrdered[] A, int lo, int cur, int hi) {
        IOrdered dat = A[cur]; // hold on to data.
        int child = 2 * cur + 1 - lo; // index of left child of A[cur].
        boolean done = hi < child;
        while (!done) {
            if (child < hi && (A[child + 1].compare(A[child])==IOrdered.GREATER)) {
                child++;
            } // child is the index of the smaller of the two children.
            if (A[child].compare(dat)==IOrdered.GREATER) {
                A[cur] = A[child];
                cur = child;
                child = 2 * cur + 1 - lo;
                done = hi < child;
            }  // parent of A[cur] < dat.
            else {// dat <= A[child], the smaller of the children of A[cur].
                done = true;  // heap condition is satisfied.
            } // A[cur] is less than its children.
        } // location found for dat.
        A[cur] = dat;
    }
}
